import 'package:flutter/material.dart';

import '../../consts.dart';

class InfoLayer extends StatefulWidget {

  const InfoLayer({
    Key? key,
  }) : super(key: key);

  @override
  InfoLayerState createState() => InfoLayerState();
}

class InfoLayerState extends State<InfoLayer> {
  @override
  void initState() {
    super.initState();
  }

  // _buildLegendItem renders a colored circle and text to form a legend
  Widget _legendItem(Color color, String text) {
    return Row(
      children: [
        Container(
          width: 20,
          height: 20,
          decoration: BoxDecoration(
            color: color,
            shape: BoxShape.circle,
          ),
        ),
        const SizedBox(width: 8),
        Text(
          text,
          style: const TextStyle(
            fontSize: 14,
            color: Colors.white,
          ),
        ),
      ],
    );
  }

  /// Builds an additional legend to explain the colors
  Widget _buildLegend() {
    return Column(
      children: [
        _legendItem(const Color(0xffff0000), "Certainly"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xffff6a00), "Very likely"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xFFb1ff00), "Very unlikely"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xFF00d6ff), "Extremely unlikely"),
        const SizedBox(height: 10),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
        child: Container(
          padding: const EdgeInsets.all(45),
          color: Colors.black.withOpacity(0.8),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center, // Align contents vertically centered
            children: [
              Text(
                'Color categorization',
                style: subHeadingStyle,
              ),
              const SizedBox(height: 20),
              Text(
                'Each category shows the likelihood of the ice breaking '
                    'under the weight of a single person.',
                style: regTextStyle,
                textAlign: TextAlign.center,
              ),
              const SizedBox(height: 30),
              _buildLegend(),
              const SizedBox(height: 30),
              Text(
                'Data accuracy',
                style: subHeadingStyle,
                textAlign: TextAlign.center,
              ),
              const SizedBox(height: 20),
              Text(
                'The higher the accuracy, the more likely it is that the displayed ice thickness'
                    'is correct. The accuracy depends on'
                    'which data sources are used to calculate the thickness.\n\n '
                    '- 1/4: only API data. \n'
                    '- 2/4: only sensor data.'
                    '- 3/4: both API and sensor data. \n'
                    '- 4/4: both API and sensor data,  with a discrepancy of <1.0cm',
                style: regTextStyle,
                textAlign: TextAlign.center,
              ),
            ],
          ),  ),
    );
  }
}
